#ifndef MiniLab_
#define MiniLab_

#include "Arduino.h"

namespace MiniLabSpace {





class Color 
{
   public:

      float r,g,b, a;

      Color (float r, float g, float b, float a);
};

extern Color red;
extern Color green;
extern Color blue;
extern Color yellow;
extern Color cyan;
extern Color magenta;
extern Color white;
extern Color black;
extern Color gray;


/// 

class _MiniLab 
{
   public:

      float readFloat ();
      int readInt ();
      String readString ();

      bool checkTitle (String title);      

      void log (String str);

      void beepSound ();   
      void asteriskSound ();
      void exclamationSound ();
      void questionSound ();
      void handSound ();


      // The special integers to or from the target

      static const int id_start                       =-1; 
         // Informing the target of the communication start, with the title to compare.
      static const int id_end                         =-2; 
         // Informing the target of the communication end.

      static const int msg_titleError                 =-1; 
         // The target is saying that its title differs from the received one.

      static const int msg_log                        =-2; 
         // The target is going to send a message.

         // The target is asking a sound.
      static const int msg_beepSound                  =-3; 
      static const int msg_asteriskSound              =-4; 
      static const int msg_exclamationSound           =-5; 
      static const int msg_questionSound              =-6; 
      static const int msg_handSound                  =-7;
};

extern _MiniLab MiniLab;


///

class Control
{
   public:

      int id;

      Control (int i);

      void enable ();
      void disable ();
      void setFront ();
      void setRear ();

   protected:

      void setControl (int work, float f1, float f2, float f3, float f4);
      void setControl (int work, float f1, float f2, float f3);
      void setControl (int work, String str);
      void setControl (int work, bool on);
      void setControl (int work, float f);
      void setControl (int work, double d);
      void setControl (int work, int i);

      // The constants about the kind of work, following the control id, when the target
      // calls(sends) a control function.
      static const int work_enable        =0;
      static const int work_disable       =1;
      static const int work_setFront      =2;
      static const int work_setRear       =3;
      static const int work_print         =4;
      static const int work_set           =5;
};

class TextControl : public Control
{
   public:

      TextControl (int i) : Control (i) { 
      }; 

      void print (String str);
      void print (float f);
      void print (double d);
      void print (int i);
};

class Label : public TextControl
{
   public:

      Label (int i) : TextControl (i) { 
      }; 
};

class TextLamp : public TextControl
{
   public:

      TextLamp (int i) : TextControl (i) {
      }; 

      void set (Color color);
};

class Push : public TextLamp
{
   public:

      Push (int i) : TextLamp (i) {
      }; 
};

class TextPanel : public TextLamp
{
   public:

      TextPanel (int i) : TextLamp (i) { 
      }; 
};

class Check : public Control
{
   public:

      Check (int i) : Control (i) { 
      }; 

      void set (bool on);
};

class Bar : public Control
{
   public:

      Bar (int i) : Control (i) { 
      }; 

      void set (float value);
};

class SlideBar : public Control
{
   public:

      SlideBar (int i) : Control (i) { 
      }; 

      void set (float minValue, float maxValue, float value);
};

class Selector : public Control
{
   public:

      Selector (int i) : Control (i) { 
      }; 

      void set (int i0_);
};

class Group : public Control
{
   public:

      Group (int i) : Control (i) { 
      }; 
};





}

using namespace MiniLabSpace;

#endif